import simplejson

from django.core.urlresolvers import reverse
from django.shortcuts import render_to_response
from django.http import HttpResponse, HttpResponseRedirect
from django.views.decorators.cache import cache_page

from mywlcg.base import release_info
from mywlcg.base.views import path_prefix
from mywlcg.filters.lib import get_vo_profile
from atp.orm.models import (Site, Groups, GroupType, VoServiceGroup,
                            Service, ServiceTypeFlavour, Infrastructure, Country)


def index(request):
    return HttpResponseRedirect(reverse('mywlcg-atp-web.views.tab', kwargs={'tab':'si'}))


def parse_option(request, option, default='', get=False, split=False):
    """Returns a value or a value list depending on the parameter type"""
    if split:
        opt = filter(lambda x: x, request.GET.get(option, default).split(','))
    else:
        opt = filter(lambda x: x, request.GET.getlist(option))
    if not len(opt):
        return default
    elif get:
        return opt[0]
    return opt


def tab(request, tab):
    if tab not in ('gr', 'si', 'se'):
        return render_to_response('../../base/templates/400.html', {'error': 'Page Does Not Exist'})
    results = {
        'tab': tab
    }

    prefix = path_prefix(request, {})
    info = release_info()
    sites = parse_option(request, 'filters-value-Physical_Sites', split=True)
    groups = parse_option(request, 'filters-value-Groups', split=True)
    gtypes = parse_option(request, 'filters-value-Group_Types', split=True)
    countries = parse_option(request, 'filters-value-Countries', split=True)
    services = parse_option(request, 'filters-value-Hostnames', split=True)
    flavours = parse_option(request, 'filters-value-Flavours', split=True)
    infrastructure = parse_option(request, 'infrastructure', get=True)
    ismonitored = parse_option(request, 'ismonitored', 'Y', get=True)
    certifstatus = parse_option(request, 'certifstatus', 'Y', get=True)
    isinprod = parse_option(request, 'isinprod', 'Y', get=True)
    order = parse_option(request, 'order', get=True)

    vos, vo, profile = get_vo_profile(request)
    infrastructures = Infrastructure.objects.filter(isdeleted='N')

    def selected_objects(model, ids, *columns):
        return model.objects.filter(id__in=ids).values_list('id', *columns)

    default_order = {
        'gr': 'service__site__sitename',
        'se': 'sitename',
        'si': 'sitename',
    }

    context = {
        'PATH': prefix['PATH'],
        'tab': tab,
        'about': info['sam_release'],
        'version': info['mywlcg_version'],

        # filter completion and user selected options
        'vos': vos,
        'vo': vo,
        'infrastructures': [(str(i.id), i.infrastname) for i in infrastructures],
        'sites': selected_objects(Site, sites, 'sitename'),
        'groups': selected_objects(Groups, groups, 'groupname'),
        'gtypes': selected_objects(GroupType, gtypes, 'typename'),
        'countries': selected_objects(Country, countries, 'countryname'),
        'services': selected_objects(Service, services, 'hostname', 'flavour__flavourname'),
        'flavour': selected_objects(ServiceTypeFlavour, flavours, 'flavourname'),

        # json api call parameters
        'filter_site': '&site='.join(sites),
        'filter_country': '&country='.join(countries),
        'filter_group': '&group='.join(groups),
        'filter_gtype': '&gtype='.join(gtypes),
        'filter_flavour': '&flavour='.join(flavours),
        'filter_service': '&service='.join(services),
        'infrastructure': infrastructure or infrastructures[0].id,
        'ismonitored': ismonitored,
        'certifstatus': certifstatus,
        'isinprod': isinprod,
        'order': order or default_order[tab],
    }
    return render_to_response('atp_index.html', context)


def get_sites(sites, countries, ismonitored, certifstatus, infra, order):
    results = {
        'data': [['Site', 'Country'], ]
    }

    filters = {
        'isdeleted': 'N',
        'ismonitored': ismonitored,
        'certifstatus': certifstatus,
        'infrast': int(infra)
    }

    if sites:
        filters['id__in'] = map(int, sites)
    if countries:
        filters['country__id__in'] = map(int, countries)

    selected_sites = Site.objects.filter(**filters).select_related(depth=1)

    allowed_orders = (
        'country__countryname', '-country__countryname',
        'sitename', '-sitename'
    )
    if order not in allowed_orders:
        order = 'sitename'
    selected_sites = selected_sites.order_by(order)

    for site in selected_sites:
        results['data'].append([site.sitename, site.country.countryname])
    return results


def get_services(vo, sites, flavours, hostnames, ismonitored, isinprod, order):
    results = {
        'data': [['Site', 'Flavour', 'Hostname'], ]
    }

    filters = {
        'isdeleted': 'N',
        'vo': vo,
        'ismonitored': ismonitored,
        'isinproduction': isinprod,
    }

    if sites:
        filters['site__in'] = map(int, sites)
    if flavours:
        filters['flavour__id__in'] = map(int, flavours)
    if hostnames:
        filters['hostname__in'] = hostnames

    selected_services = Service.objects.filter(**filters).select_related(depth=2)

    allowed_orders = (
        'site__sitename', '-site__sitename',
        'flavour__flavourname', '-flavour__flavourname',
        'hostname', '-hostname'
    )
    if order not in allowed_orders:
        order = ('site__sitename')
    selected_services = selected_services.order_by(order)

    for service in selected_services:
        results['data'].append([service.site.get().sitename, service.flavour.flavourname, service.hostname])
    return results


def get_groups(vo, sites, groups, gtypes, flavours, services, order):
    results = {
        'data': [['Group', 'Group type', 'Site', 'Flavour', 'Hostname'], ]
    }

    filters = {
        'isdeleted': 'N',
        'vo': vo,
    }

    if sites:
        filters['service__site__in'] = map(int, sites)
    if groups:
        filters['groups__in'] = map(int, groups)
    if gtypes:
        filters['groups__group_type__in'] = map(int, gtypes)
    if flavours:
        filters['service__flavour__in'] = map(int, flavours)
    if services:
        filters['service__in'] = map(int, services)

    selected_groups = VoServiceGroup.objects.filter(**filters).select_related(depth=2)
    allowed_orders = (
        'groups__groupname', '-groups__groupname',
        'groups__group_type__typename', '-groups__group_type__typename',
        'service__site__sitename', '-service__site__sitename',
        'flavour__flavourname', '-flavour__flavourname',
        'service__hostname', '-service__hostname'
    )
    if order not in allowed_orders:
        order = 'groups__groupname'
    selected_groups = selected_groups.order_by(order)

    for group in selected_groups:
        results['data'].append([
            group.groups.groupname, group.groups.group_type.typename,
            group.service.site.get().sitename, group.service.flavour.flavourname,
            group.service.hostname])
    return results


@cache_page(3600)
def json(request, tab):
    sites = parse_option(request, 'site')
    groups = parse_option(request, 'group')
    gtypes = parse_option(request, 'gtype')
    countries = parse_option(request, 'country')
    hostnames = parse_option(request, 'hostnames')
    services = parse_option(request, 'services')
    flavours = parse_option(request, 'flavour')
    infra = parse_option(request, 'infrastructure', get=True)
    vo = parse_option(request, 'vo', get=True)
    ismonitored = parse_option(request, 'ismonitored', 'Y', get=True)
    certifstatus = parse_option(request, 'certifstatus', 'Y', get=True)
    isinprod = parse_option(request, 'isinprod', 'Y', get=True)
    order = parse_option(request, 'order', get=True)
    callback = 'atp_data_json'

    if tab == "si":
        results = get_sites(sites, countries, ismonitored, certifstatus, infra, order)

    elif tab == "se":
        results = get_services(vo, sites, flavours, hostnames, ismonitored, isinprod, order)

    elif tab == "gr":
        results = get_groups(vo, sites, groups, gtypes, flavours, services, order)

    return HttpResponse(simplejson.dumps({'aaData':results['data']}), mimetype="application/json")

